#region Copyright
// 
// DotNetNuke - http://www.dotnetnuke.com
// Copyright (c) 2002-2012
// by DotNetNuke Corporation
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated 
// documentation files (the "Software"), to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and 
// to permit persons to whom the Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions 
// of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED 
// TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF 
// CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
#endregion
#region Usings

using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Threading;
using System.Web;
using System.Linq;

using DotNetNuke.Common;
using DotNetNuke.Common.Utilities;
using DotNetNuke.Entities.Controllers;
using DotNetNuke.Entities.Host;
using DotNetNuke.Entities.Portals;
using DotNetNuke.Entities.Tabs;
using DotNetNuke.HttpModules.Config;
using DotNetNuke.Instrumentation;
using DotNetNuke.Services.ClientCapability;
using DotNetNuke.Services.EventQueue;
using DotNetNuke.Services.Localization;
using Satrabel.HttpModules.Config;
using Satrabel.HttpModules.Provider;
using Satrabel.Services.Log.UrlLog;
using DotNetNuke.Entities.Users;

#endregion

namespace Satrabel.HttpModules
{
    public class UrlRewriteModule : IHttpModule
    {
        public string ModuleName
        {
            get
            {
                return "UrlRewriteModule";
            }
        }

        #region IHttpModule Members

        public void Init(HttpApplication application)
        {
            
            application.BeginRequest += OnBeginRequest;
            UrlRewriterLogging logger = new UrlRewriterLogging();
            application.EndRequest += logger.OnEndRequest;
            application.PreRequestHandlerExecute += logger.OnPreRequestHandlerExecute;
            //application.Error += OnError;

            
        }

        public void Dispose()
        {
        }

        #endregion

        private string FormatDomain(string url, string replaceDomain, string withDomain)
        {
            if (!String.IsNullOrEmpty(replaceDomain) && !String.IsNullOrEmpty(withDomain))
            {
                if (url.IndexOf(replaceDomain) != -1)
                {
                    url = url.Replace(replaceDomain, withDomain);
                }
            }
            return url;
        }

        private void RewriteUrl(HttpApplication app, out string portalAlias)
        {
            HttpRequest request = app.Request;
            HttpResponse response = app.Response;            
            portalAlias = "";
            //determine portal alias looking for longest possible match
            PortalAliasInfo objPortalAlias = GetPortalAlias(app, out portalAlias);
            //String myAlias = portalAlias;
            // save OriginalUrl & PortalId for Logging
            app.Context.Items.Add("UrlRewrite:OriginalUrl", app.Request.Url.AbsoluteUri);
            app.Context.Items.Add("UrlRewrite:PortalId", (objPortalAlias == null ? -1 : objPortalAlias.PortalID) );
           
            //string MyLocalPath = app.Request.Url.LocalPath;
            // action is the object containing all info about rewiting and redirection
            RewriterAction action = new RewriterAction();
            action.LocalPath = app.Request.Url.LocalPath;
            action.HostPort = GetHostPort(app);
            action.OriginalUrl = action.HostPort + action.LocalPath;
            action.Alias = portalAlias;
            action.QueryUrl = app.Request.Url.Query;
            //action.RedirectUrl = action.OriginalUrl;

            // check for language as parameter in the url (OpenUrlRewriter use for public urls, the language after the portal alias)
            Match langMatch = Regex.Match(action.LocalPath, "/language/(?:.[^/]+)(?:/|$)", RegexOptions.IgnoreCase); //searches for a string like language/en-US/ in the url
            
            if (langMatch.Success)
            {
                string langParms = langMatch.Value.TrimEnd('/');//in the format of /language/en-US only                
                action.CultureCode = langParms.ToLower().Replace("/language/", "");
                // construct the standard dnn path format /alias/culture/rest of the path
                action.LocalPath = app.Context.Request.ApplicationPath + "/" + action.CultureCode + action.LocalPath.Substring(app.Context.Request.ApplicationPath.Length).Replace(langParms, "");
                //langParms = langParms.ToLower().Replace("/language", "");
                //action.RedirectUrl = action.HostPort + action.LocalPath;
                action.DoRedirect = true;
                action.Raison += "+language present";
            }

            bool RuleMatch = false;
            //int tabID = Null.NullInteger;
            // check for tabid in the url (OpenUrlRewriter use for public urls, the pagename after the language in the url)
            string pattern = "^" + RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, "[^?]*/TabId/(\\d+)(.*)") + "$";
            Match objMatch = Regex.Match(app.Request.Url.LocalPath, pattern, RegexOptions.IgnoreCase);
            //if there is a match
            if (objMatch.Success)
            {
                action.TabId = int.Parse(objMatch.Groups[1].Value);
                // if there is a rule for the tabid, the OpenUrlRewriter manage the rewriter otherwise the SiteUrls rules discard all processing
                var rule = GetRewriteTabRule(objPortalAlias.PortalID, action.CultureCode, action.TabId);
                if (rule != null){
                    // construct the standard dnn path format /alias/culture/pageurl/parameters
                    string langParms = string.IsNullOrEmpty(action.CultureCode) ? "" : "/" + action.CultureCode;
                    action.LocalPath = RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, Regex.Replace(action.LocalPath, pattern, "~"+ langParms +"/"+ rule.Url + "$2", RegexOptions.IgnoreCase));
                    //action.RedirectUrl = action.HostPort + action.LocalPath;
                    action.DoRedirect = true;
                    action.Raison += "+tabid present";
                    RuleMatch = true;
                }
                //  ~/Default.aspx?TabId=$1                
            }                   
            // Process SiteUrls.config rules (for exemple url with tabid)           
            if (!RuleMatch)
            {
                if (ProcessRules(app, objPortalAlias)) return;
            }            
            // custom redirects            
            if (ProcessCustomRules(app, objPortalAlias)) return;            

            /*
            Globals.SetApplicationName(result.portalId);
            // load the PortalSettings into current context 
            portalSettings = new PortalSettings(result.tabId, result.portalAlias);
            */

            string url = action.HostPort + action.LocalPath;
            if (!String.IsNullOrEmpty(action.Alias) && objPortalAlias != null)
            {                        
                int portalID = objPortalAlias.PortalID;
                //Work variable
                action.WorkUrl = url;
                if (action.WorkUrl.StartsWith(action.Alias))
                {
                    action.WorkUrl = url.Remove(0, action.Alias.Length);
                }                
                Dictionary<string, Locale> dicLocales = LocaleController.Instance.GetLocales(portalID);                
                //Default Page has been Requested
                if ((action.WorkUrl == "/" + Globals.glbDefaultPage.ToLower()))
                {
                    Locale DefaultLocale = LocaleController.Instance.GetDefaultLocale(portalID);
                    PortalInfo objPortal = new PortalController().GetPortal(portalID, DefaultLocale.Code);
                    string sendToUrl = "~/" + Globals.glbDefaultPage +"?TabID=" + objPortal.HomeTabId;
                    if (dicLocales.Count > 1)
                    {
                        sendToUrl = sendToUrl + "&language=" + objPortal.DefaultLanguage;
                    }
                    app.Context.Items.Add("UrlRewrite:RewriteUrl", RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, sendToUrl));
                    RewriterUtils.RewriteUrl(app.Context, sendToUrl);
                    return;
                }
                // Find culture                
                //string cultureUrl;
                GetCulture(portalID, dicLocales, action);              
                // remove default.aspx
                //string TabPath = RemoveDefaultAndExtention(action);

                RemoveDefaultAndExtention(action);


                string parameters = "";
                //String myTabPath = TabPath;
                // Find tabid
                GetTab(portalID, action);

                app.Context.Response.AppendHeader("X-OpenUrlRewriter-Info", action.CultureUrl+"*"+action.PageUrl+"*"+action.ModuleUrl);
                
                /* old stuff find tab by path (standard dnn)
                if (tabID == Null.NullInteger)
                {
                    myTabPath = TabPath;
                    tabID = GetTabByTabPath(portalID, tabPath, cultureCode);
                }
                 */

                if ((action.TabId != Null.NullInteger))
                {
                    // check for if we redirect to lower case
                    //ProcessLowerCaseRedirect(portalID, app);
                    string sendToUrl = "~/" + Globals.glbDefaultPage + "?TabID=" + action.TabId;
                    if (!string.IsNullOrEmpty(action.CultureCode))
                    {
                        sendToUrl = sendToUrl + "&language=" + action.CultureCode;
                    }
                    
                    //ProcessParametersRedirects(app, response, portalID, cultureCode, myAlias, cultureUrl, myTabPath, parameters, redirect);
                    //process the parameters
                    ProcessParameters(portalID, action);
                    if (!string.IsNullOrEmpty(action.ModuleParameters))
                    {
                        sendToUrl += "&"+ action.ModuleParameters;
                    }
                    if (!string.IsNullOrEmpty(action.QueryParameters))
                    {
                        sendToUrl += action.QueryParameters;
                    }

                    if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
                    {
                        sendToUrl = sendToUrl + "&" + app.Request.Url.Query.TrimStart('?');
                    }
                    if (ProcessRedirect(app, response, action)) return;
                    app.Context.Items.Add("UrlRewrite:RewriteUrl", RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, sendToUrl));
                    RewriterUtils.RewriteUrl(app.Context, sendToUrl);
                    return;
                }
                else 
                {                    
                    // tabid is retrived from UrlRule
                    ProcessParametersWithoutPage(portalID, action);
                    if (action.TabId != -1)
                    {
                        string sendToUrl = "~/" + Globals.glbDefaultPage + "?TabID=" + action.TabId;
                        if (!string.IsNullOrEmpty(action.CultureCode))
                        {
                            sendToUrl += "&language=" + action.CultureCode;
                        }
                        if (!string.IsNullOrEmpty(action.ModuleParameters))
                        {
                            sendToUrl += "&"+action.ModuleParameters;
                        }
                        //ProcessLowerCaseRedirect(portalID, app);
                        if (ProcessRedirect(app, response, action)) return;
                        if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
                        {
                            sendToUrl = sendToUrl + "&" + app.Request.Url.Query.TrimStart('?');
                        }
                        if (ProcessRedirect(app, response, action)) return;
                        app.Context.Items.Add("UrlRewrite:RewriteUrl", RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, sendToUrl));
                        RewriterUtils.RewriteUrl(app.Context, sendToUrl);
                        return;
                    }
                }
                // urls not included in the UrlRules                                                       
                string tabPath = "/" + action.WorkUrl.ToLower();
                if ((tabPath.IndexOf('?') != -1))
                {
                    tabPath = tabPath.Substring(0, tabPath.IndexOf('?'));
                }
                // process special urls login, register, terms, privacy
                if (ProccessCtl(app, portalID, tabPath)){
                    return;
                }
                tabPath = tabPath.Replace("/", "//");
                var objTabController = new TabController();
                TabCollection objTabs;
                if (tabPath.StartsWith("//host"))
                {
                    objTabs = objTabController.GetTabsByPortal(Null.NullInteger);
                }
                else
                {
                    objTabs = objTabController.GetTabsByPortal(portalID);
                }
                foreach (KeyValuePair<int, TabInfo> kvp in objTabs)
                {
                    if ((kvp.Value.IsDeleted == false && kvp.Value.TabPath.ToLower() == tabPath))
                    {
                        if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
                        {
                            RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + kvp.Value.TabID + "&" + app.Request.Url.Query.TrimStart('?'));
                        }
                        else
                        {
                            RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + kvp.Value.TabID);
                        }
                        return;
                    }
                }                
            }
            else
            {
                //Should always resolve to something
                //RewriterUtils.RewriteUrl(app.Context, "~/" & glbDefaultPage)
                return;
            }
        
        }

        private static string GetHostPort(HttpApplication app)
        {
            string MyHostPort;
            if (Globals.UsePortNumber() && ((app.Request.Url.Port != 80 && !app.Request.IsSecureConnection) || (app.Request.Url.Port != 443 && app.Request.IsSecureConnection)))
            {
                MyHostPort = app.Request.Url.Host + ":" + app.Request.Url.Port;
            }
            else
            {
                MyHostPort = app.Request.Url.Host;
            }
            return MyHostPort;
        }

        private static void RemoveDefaultAndExtention(RewriterAction action )
        {
            string Ext = "";
            if (action.WorkUrl.Contains('.')) {
                int ExtIndex = action.WorkUrl.IndexOf('.');
                Ext = action.WorkUrl.Substring(ExtIndex);
                action.WorkUrl = action.WorkUrl.Remove(ExtIndex);
            } 
            else if (action.WorkUrl.EndsWith("/"))
            {
                Ext = "/";
                action.WorkUrl = action.WorkUrl.TrimEnd('/');
            }
            

            if ( Ext != UrlRewiterSettings.Current().FileExtension)
            {
                //action.RedirectUrl = action.RedirectUrl.Replace(".aspx", "");
                action.DoRedirect = true;
                action.Raison += "+wrong extention";                
            }


            if (action.WorkUrl.EndsWith("/Default", StringComparison.InvariantCultureIgnoreCase))
            {
                //action.RedirectUrl = action.RedirectUrl.Replace("/Default", "");
                action.DoRedirect = true;
                action.Raison += "+Remove /Default";

                action.WorkUrl = action.WorkUrl.Remove(action.WorkUrl.Length-8);
            }
            action.WorkUrl = action.WorkUrl.TrimStart('/');
        }

        private static bool ProcessRedirect(HttpApplication app, HttpResponse response, RewriterAction redirect)
        {
            if (redirect.DoRedirect && app.Request.HttpMethod == "GET")
            {
                string RedirectTo = app.Request.Url.Scheme + "://" + redirect.Alias;
                
                if (!redirect.RedirectHomePage)
                {
                    if (!string.IsNullOrEmpty(redirect.RedirectCulture))
                    {
                        RedirectTo += "/" + redirect.RedirectCulture;
                    }
                    else if (!string.IsNullOrEmpty(redirect.CultureUrl))
                    {
                        RedirectTo += "/" + redirect.CultureUrl;
                    }
                    if (!string.IsNullOrEmpty(redirect.RedirectPage))
                    {
                        RedirectTo += "/" + redirect.RedirectPage;
                    }
                    else
                    {
                        RedirectTo += "/" + redirect.PageUrl;
                    }
                    if (!string.IsNullOrEmpty(redirect.RedirectModule))
                    {
                        RedirectTo += "/" + redirect.RedirectModule;
                    }
                    else if (!string.IsNullOrEmpty(redirect.ModuleUrl))
                    {
                        RedirectTo += "/" + redirect.ModuleUrl;
                    }

                    RedirectTo += UrlRewiterSettings.Current().FileExtension;
                    RedirectTo += app.Request.Url.Query;
                }

                
                app.Context.Items.Add("UrlRewrite:RedirectUrl", RedirectTo);

                response.AppendHeader("X-Redirect-Raison", redirect.Raison);
                if (redirect.Status == 302)
                {
                    response.Redirect(RedirectTo, true);
                }
                else
                {                    
                    response.Status = "301 Moved Permanently";
                    response.AddHeader("Location", RedirectTo);
                    response.End();
                }
                // normaly we dont come here because redirect && End stop the thread
                return true;
            }
            return false;
        }

        private void ProcessLowerCaseRedirect(int PortalId, HttpApplication app)
        {
            // redirect if requested path is not lowercase

            string requestedPath = app.Request.Url.AbsoluteUri;
            string strQueryString = "";
            if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
            {
                strQueryString = app.Request.QueryString.ToString();
                requestedPath = requestedPath.Replace(app.Request.Url.Query, "");
            }
            if (UrlRewiterSettings.IsUrlToLowerCase() && 
                !UrlRewiterSettings.ExcludeFromRedirect(PortalId, requestedPath) && 
                !UrlRewiterSettings.ExcludeFromLowerCase(PortalId, requestedPath))
            {
                
                
                if (requestedPath != requestedPath.ToLower())
                {
                    string sendTo = requestedPath.ToLower();
                    if (strQueryString != "")
                    {
                        sendTo = sendTo + "?" + strQueryString;
                    }

                    app.Context.Items.Add("UrlRewrite:RedirectUrl", sendTo);
                    app.Response.Status = "301 Moved Permanently";
                    app.Response.AppendHeader("X-Redirect-Reason", "Not lowercase");
                    app.Response.AddHeader("Location", sendTo);
                    app.Response.End();
                }
            }
        }

        /*
        private void ProcessLowerCaseRedirect(int PortalId, HttpApplication app, RewriterAction action)
        {
            // redirect if requested path is not lowercase
            
            if (UrlRewiterSettings.IsUrlToLowerCase() &&
                !UrlRewiterSettings.ExcludeFromRedirect(PortalId, requestedPath) &&
                !UrlRewiterSettings.ExcludeFromLowerCase(PortalId, requestedPath))
            {


                if (requestedPath != requestedPath.ToLower())
                {
                    string sendTo = requestedPath.ToLower();
                    if (strQueryString != "")
                    {
                        sendTo = sendTo + "?" + strQueryString;
                    }

                    app.Context.Items.Add("UrlRewrite:RedirectUrl", sendTo);
                    app.Response.Status = "301 Moved Permanently";
                    app.Response.AppendHeader("X-Redirect-Reason", "Not lowercase");
                    app.Response.AddHeader("Location", sendTo);
                    app.Response.End();
                }
            }
        }
        */
        private static int GetTabByTabPath(int portalID, string tabPath, string cultureCode)
        {
            // Check to see if the tab exists (if localization is enable, check for the specified culture)
            int tabID = TabController.GetTabByTabPath(portalID, tabPath.Replace("/", "//").Replace(".aspx", ""), cultureCode);

            // Check to see if neutral culture tab exists
            if ((tabID == Null.NullInteger && cultureCode.Length > 0))
            {
                tabID = TabController.GetTabByTabPath(portalID, tabPath.Replace("/", "//").Replace(".aspx", ""), "");
            }
            return tabID;
        }

        private bool ProccessCtl(HttpApplication app, int portalID, string tabPath) {

            //Get the Portal
            PortalInfo portal = new PortalController().GetPortal(portalID);
            var requestQuery = app.Request.Url.Query;
            if (!string.IsNullOrEmpty(requestQuery))
            {
                requestQuery = Regex.Replace(requestQuery, "&?tabid=\\d+", string.Empty, RegexOptions.IgnoreCase);
                requestQuery = Regex.Replace(requestQuery, "&?portalid=\\d+", string.Empty, RegexOptions.IgnoreCase);
                requestQuery = requestQuery.TrimStart('?', '&');
            }

            tabPath = tabPath.Replace(".aspx", "");
            if (tabPath == "/login")
            {
                ProcessLowerCaseRedirect(portalID, app);
                if (portal.LoginTabId > Null.NullInteger && Globals.ValidateLoginTabID(portal.LoginTabId))
                {
                    if (!string.IsNullOrEmpty(requestQuery))
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.LoginTabId + "&" + requestQuery);
                    }
                    else
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.LoginTabId);
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(requestQuery))
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=login&" + requestQuery);
                    }
                    else
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=login");
                    }
                }
                return true;
            }
            if (tabPath == "/register")
            {
                ProcessLowerCaseRedirect(portalID, app);
                if (portal.RegisterTabId > Null.NullInteger)
                {
                    if (!string.IsNullOrEmpty(requestQuery))
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.RegisterTabId + "&portalid=" + portalID + "&" + requestQuery);
                    }
                    else
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.RegisterTabId + "&portalid=" + portalID);
                    }
                }
                else
                {
                    if (!string.IsNullOrEmpty(requestQuery))
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Register&" + requestQuery);
                    }
                    else
                    {
                        RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Register");
                    }
                }
                return true;
            }
            if (tabPath == "/terms")
            {
                ProcessLowerCaseRedirect(portalID, app);
                if (!string.IsNullOrEmpty(requestQuery))
                {
                    RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Terms&" + requestQuery);
                }
                else
                {
                    RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Terms");
                }
                return true;
            }
            if (tabPath == "/privacy")
            {
                ProcessLowerCaseRedirect(portalID, app);
                if (!string.IsNullOrEmpty(requestQuery))
                {
                    RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Privacy&" + requestQuery);
                }
                else
                {
                    RewriterUtils.RewriteUrl(app.Context, "~/" + Globals.glbDefaultPage + "?TabID=" + portal.HomeTabId + "&portalid=" + portalID + "&ctl=Privacy");
                }
                return true;
            }

            return false;
        }


        private static void GetCulture(int portalID, Dictionary<string, Locale> dicLocales, RewriterAction action)
        {
            string cultureCode = string.Empty;
            //cultureUrl = string.Empty;
            // check if multi language site
            if (dicLocales.Count > 1)
            {
                String[] splitUrl = action.WorkUrl.TrimStart('/').Split('/');
                // asume culture is always first part of url afetr alias
                string culturePart = splitUrl[0];
                // try to find UrlRule                            
                var ruleL = GetLanguageRule(portalID, culturePart.ToLower());
                if (ruleL != null)
                {                    
                    if (ruleL.Action == UrlRuleAction.Redirect) 
                    {                        
                        //action.RedirectUrl = redirect.RedirectUrl.Replace("/" + culturePart + "/", "/" + ruleL.RedirectDestination + "/");
                        action.RedirectCulture = ruleL.RedirectDestination;
                        action.Status = ruleL.RedirectStatus;
                        action.DoRedirect = true;
                        action.Raison += "+Culture:" + ruleL.Url + ">" + ruleL.RedirectDestination;
                    }
                    else if (culturePart != ruleL.Url) // because different case
                    {
                        action.RedirectPage = ruleL.Url;
                        action.DoRedirect = true;
                        action.Raison += "+Wrong case";
                    }
                    culturePart = ruleL.CultureCode;
                }

                foreach (KeyValuePair<string, Locale> key in dicLocales)
                {
                    if (key.Key.ToLower().Equals(culturePart.ToLower()))
                    {
                        action.CultureCode = key.Value.Code;
                        action.CultureUrl = splitUrl[0];
                        action.WorkUrl = action.WorkUrl.Substring(action.CultureUrl.Length + 1);
                        break;
                    }
                }
            }
        }

        private static bool ProcessRules(HttpApplication app, PortalAliasInfo objPortalAlias)
        {
            //Friendly URLs are exposed externally using the following format
            //http://www.domain.com/tabid/###/mid/###/ctl/xxx/default.aspx
            //and processed internally using the following format
            //http://www.domain.com/default.aspx?tabid=###&mid=###&ctl=xxx
            //The system for accomplishing this is based on an extensible Regex rules definition stored in /SiteUrls.config


            //save and remove the querystring as it gets added back on later
            //path parameter specifications will take precedence over querystring parameters
            string requestedPath = app.Request.Url.AbsoluteUri;
            string strQueryString = "";
            if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
            {
                strQueryString = app.Request.QueryString.ToString();
                requestedPath = requestedPath.Replace(app.Request.Url.Query, "");
            }


            string sendTo = "";
            //get url rewriting rules 
            RewriterRuleCollection rules = RewriterConfiguration.GetConfig().Rules;
                                    
            //iterate through list of rules
            int matchIndex = -1;
            for (int ruleIndex = 0; ruleIndex <= rules.Count - 1; ruleIndex++)
            {
                //check for the existence of the LookFor value 
                string pattern = "^" + RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, rules[ruleIndex].LookFor) + "$";
                Match objMatch = Regex.Match(requestedPath, pattern, RegexOptions.IgnoreCase);

                //if there is a match
                if ((objMatch.Success))
                {
                    //create a new URL using the SendTo regex value
                    sendTo = RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, Regex.Replace(requestedPath, pattern, rules[ruleIndex].SendTo, RegexOptions.IgnoreCase));

                    string parameters = objMatch.Groups[2].Value;
                    //process the parameters
                    //ProcessParameters(objPortalAlias.PortalID, "", -1, "",ref sendTo, parameters, null);
                    if ((parameters.Trim().Length > 0))
                    {
                        //split the value into an array based on "/" ( ie. /tabid/##/ )
                        parameters = parameters.Replace("\\", "/");

                        parameters = parameters.Replace("/" + Globals.glbDefaultPage, "").Replace("/" + Globals.glbDefaultPage.ToLower(), "").Replace(".aspx", "");

                        string[] splitParameters = parameters.Split('/');
                        string parameterName;
                        string parameterValue;
                        //icreate a well formed querystring based on the array of parameters
                        for (int parameterIndex = 0; parameterIndex < splitParameters.Length; parameterIndex++)
                        {
                            //ignore the page name 
                            if (splitParameters[parameterIndex].IndexOf(".aspx", StringComparison.InvariantCultureIgnoreCase) == -1)
                            {
                                //get parameter name
                                parameterName = splitParameters[parameterIndex].Trim();
                                if (parameterName.Length > 0)
                                {
                                    //add parameter to SendTo if it does not exist already  
                                    if (sendTo.IndexOf("?" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1 &&
                                        sendTo.IndexOf("&" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1)
                                    {
                                        //get parameter delimiter
                                        string parameterDelimiter;
                                        if (sendTo.IndexOf("?") != -1)
                                        {
                                            parameterDelimiter = "&";
                                        }
                                        else
                                        {
                                            parameterDelimiter = "?";
                                        }
                                        sendTo = sendTo + parameterDelimiter + parameterName;
                                        //get parameter value
                                        parameterValue = "";
                                        if (parameterIndex < splitParameters.Length - 1)
                                        {
                                            parameterIndex += 1;
                                            if (!String.IsNullOrEmpty(splitParameters[parameterIndex].Trim()))
                                            {
                                                parameterValue = splitParameters[parameterIndex].Trim();
                                            }
                                        }
                                        //add the parameter value
                                        if (parameterValue.Length > 0)
                                        {
                                            sendTo = sendTo + "=" + parameterValue;
                                        }
                                    }
                                }
                            }
                        }
                    }
                    matchIndex = ruleIndex;
                    break; //exit as soon as it processes the first match
                }
            }


            //if a match was found to the urlrewrite rules
            if (matchIndex != -1)
            {
                if (!String.IsNullOrEmpty(strQueryString))
                {
                    //add querystring parameters back to SendTo
                    string[] parameters = strQueryString.Split('&');
                    string parameterName;
                    //iterate through the array of parameters
                    for (int parameterIndex = 0; parameterIndex <= parameters.Length - 1; parameterIndex++)
                    {
                        //get parameter name
                        parameterName = parameters[parameterIndex];
                        if (parameterName.IndexOf("=") != -1)
                        {
                            parameterName = parameterName.Substring(0, parameterName.IndexOf("="));
                        }
                        //check if parameter already exists
                        if (sendTo.IndexOf("?" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1 &&
                            sendTo.IndexOf("&" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1)
                        {
                            //add parameter to SendTo value
                            if (sendTo.IndexOf("?") != -1)
                            {
                                sendTo = sendTo + "&" + parameters[parameterIndex];
                            }
                            else
                            {
                                sendTo = sendTo + "?" + parameters[parameterIndex];
                            }
                        }
                    }
                }

                if (rules[matchIndex].SendTo.StartsWith("~"))
                {
                    //rewrite the URL for internal processing
                    RewriterUtils.RewriteUrl(app.Context, sendTo);
                }
                else
                {
                    //it is not possible to rewrite the domain portion of the URL so redirect to the new URL
                    app.Context.Items.Add("UrlRewrite:RedirectUrl", sendTo);
                    app.Response.Redirect(sendTo, true);
                }
                return true;
            } else {
                return false;
            }

            
        }


        private static bool ProcessCustomRules(HttpApplication app, PortalAliasInfo objPortalAlias)
        {
            //save and remove the querystring as it gets added back on later
            //path parameter specifications will take precedence over querystring parameters
            string requestedPath = app.Request.Url.AbsoluteUri;
            string strQueryString = "";
            if ((!String.IsNullOrEmpty(app.Request.Url.Query)))
            {
                strQueryString = app.Request.QueryString.ToString();
                requestedPath = requestedPath.Replace(app.Request.Url.Query, "");
            }


            string sendTo = "";
            //get url rewriting rules 
            //RewriterRuleCollection rules = RewriterConfiguration.GetConfig().Rules;
            var rules = GetRules(objPortalAlias.PortalID).Where(r => r.RuleType == UrlRuleType.Custom);

            //iterate through list of rules
            UrlRule ruleFound = null;
            foreach (UrlRule rule in rules)
            {
                //check for the existence of the LookFor value 
                string pattern = "^" + RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, rule.Url) + "$";
                Match objMatch = Regex.Match(requestedPath, pattern, RegexOptions.IgnoreCase);

                //if there is a match
                if ((objMatch.Success))
                {
                    //create a new URL using the SendTo regex value
                    sendTo = RewriterUtils.ResolveUrl(app.Context.Request.ApplicationPath, Regex.Replace(requestedPath, pattern, rule.RedirectDestination, RegexOptions.IgnoreCase));

                    string parameters = objMatch.Groups[2].Value;
                    //process the parameters
                    //ProcessParameters(objPortalAlias.PortalID, "", -1,  ref sendTo, parameters);
                    ruleFound = rule;
                    break; //exit as soon as it processes the first match
                }
            }

            //if a match was found to the urlrewrite rules
            if (ruleFound != null)
            {
                if (!String.IsNullOrEmpty(strQueryString))
                {
                    //add querystring parameters back to SendTo
                    string[] parameters = strQueryString.Split('&');
                    string parameterName;
                    //iterate through the array of parameters
                    for (int parameterIndex = 0; parameterIndex <= parameters.Length - 1; parameterIndex++)
                    {
                        //get parameter name
                        parameterName = parameters[parameterIndex];
                        if (parameterName.IndexOf("=") != -1)
                        {
                            parameterName = parameterName.Substring(0, parameterName.IndexOf("="));
                        }
                        //check if parameter already exists
                        if (sendTo.IndexOf("?" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1 &&
                            sendTo.IndexOf("&" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1)
                        {
                            //add parameter to SendTo value
                            if (sendTo.IndexOf("?") != -1)
                            {
                                sendTo = sendTo + "&" + parameters[parameterIndex];
                            }
                            else
                            {
                                sendTo = sendTo + "?" + parameters[parameterIndex];
                            }
                        }
                    }
                }
                app.Context.Items.Add("UrlRewrite:RedirectUrl", sendTo);

                if (ruleFound.Action == UrlRuleAction.Rewrite)
                {
                    RewriterUtils.RewriteUrl(app.Context, sendTo);
                }
                else
                {
                    if (ruleFound.RedirectStatus == 302)
                    {
                        app.Response.Redirect(sendTo, true);
                    }
                    else
                    {
                        app.Response.Status = "301 Moved Permanently";
                        app.Response.AppendHeader("X-Redirect-Reason", "Cusiom rule");
                        app.Response.AddHeader("Location", sendTo);
                        app.Response.End();
                    }
                }
                return true;
            }
            else
            {
                return false;
            }
        }
        /*
        private static void ProcessParametersRedirects(HttpApplication app, HttpResponse response, int portalId, string CultureCode, String myAlias, string cultureUrl, string tabUrl, string parameters, RedirectAction redirect)
        {
            if ((parameters.Trim().Length > 0))
            {
                UrlRule CtlRule = GetRedirectModuleRule(portalId, CultureCode, parameters.TrimStart('/').ToLower());
                if (CtlRule != null)
                {
                    if (CtlRule.RemoveTab)
                    {
                        redirect.RedirectUrl = redirect.RedirectUrl.Replace("/" + tabUrl, "");
                    }
                    redirect.RedirectUrl = redirect.RedirectUrl.Replace(parameters.TrimStart('/'), CtlRule.RedirectDestination);
                    redirect.DoRedirect = true;
                    redirect.Raison += "+ModuleRule:" + CtlRule.Url + ">" + CtlRule.RedirectDestination;
                }
            }
        }
        */
        private static void GetTab(int portalId, RewriterAction action)
        {
            int tabID = Null.NullInteger;
            string TabPath = action.WorkUrl;
            string Parameters = "";
            UrlRule TabRule;
            do
            {
                TabRule = GetTabRule(portalId, action.CultureCode, TabPath);
                if (TabRule != null)
                {
                    action.TabId = TabRule.TabId;
                    action.WorkUrl = "";
                    action.PageUrl = TabPath;
                    action.ModuleUrl = Parameters.TrimStart('/');

                    if (TabRule.RemoveTab && string.IsNullOrEmpty(action.ModuleUrl) && string.IsNullOrEmpty(action.QueryUrl))
                    { // redirect to alias
                        action.RedirectHomePage = true;
                        action.Status = TabRule.RedirectStatus;
                        action.DoRedirect = true;
                        action.Raison += "+Tab:" + TabRule.Url + "> alias";
                    }
                    else if (TabRule.Action == UrlRuleAction.Redirect)
                    {
                        //action.RedirectUrl = action.RedirectUrl.Replace("/" + TabRule.Url, "/" + TabRule.RedirectDestination);
                        action.RedirectPage = TabRule.RedirectDestination;
                        action.Status = TabRule.RedirectStatus;
                        action.DoRedirect = true;
                        action.Raison += "+Tab:" + TabRule.Url + ">" + TabRule.RedirectDestination;
                        
                    }
                    else if (TabPath != TabRule.Url) // because different case
                    {
                        action.RedirectPage = TabRule.Url;
                        action.DoRedirect = true;
                        action.Raison += "+Wrong case";
                    }
                    break;
                }
                
                int slashIndex = TabPath.LastIndexOf('/');
                if (slashIndex > 0)
                {
                    Parameters = TabPath.Substring(slashIndex) + Parameters;
                    TabPath = TabPath.Substring(0, slashIndex);
                }
                else
                {
                    TabPath = "";
                }
            } while (TabPath.Length > 0);


            
        }


        private static PortalAliasInfo GetPortalAlias(HttpApplication app, out string portalAlias)
        {
            PortalAliasInfo objPortalAlias = null;
            string myAlias = Globals.GetDomainName(app.Request, true);
            portalAlias = "";
            do
            {
                objPortalAlias = PortalAliasController.GetPortalAliasInfo(myAlias);

                if (objPortalAlias != null)
                {
                    portalAlias = myAlias;
                    break;
                }

                int slashIndex = myAlias.LastIndexOf('/');
                if (slashIndex > 1)
                {
                    myAlias = myAlias.Substring(0, slashIndex);
                }
                else
                {
                    myAlias = "";
                }
            } while (myAlias.Length > 0);

            return objPortalAlias;
        }

        private static void ProcessParameters(int portalId, RewriterAction action)
        {
            if ((action.ModuleUrl.Trim().Length > 0))
            {
                //split the value into an array based on "/" ( ie. /tabid/##/ )
                string parameters = action.ModuleUrl.Replace("\\", "/").TrimStart('/');
                var rule = GetModuleRule(portalId, action.CultureCode, action.TabId, parameters);
                              
                if (rule != null) {
                    if (rule.Action == UrlRuleAction.Redirect)
                    {
                        if (rule.RemoveTab)
                        {
                            //action.RedirectUrl = redirect.RedirectUrl.Replace("/" + tabUrl, "");
                            action.RedirectPage = "";
                        }
                        //action.RedirectUrl = redirect.RedirectUrl.Replace(parameters.TrimStart('/'), rule.RedirectDestination);
                        action.RedirectModule = rule.RedirectDestination;
                        action.DoRedirect = true;
                        action.Raison += "+ModuleRule:" + rule.Url + ">" + rule.RedirectDestination;

                    }
                    else if (parameters != rule.Url) // because different case
                    {
                        action.RedirectPage = rule.Url;
                        action.DoRedirect = true;
                        action.Raison += "+Wrong case";
                    }
                    action.ModuleParameters = rule.Parameters;
                    action.WorkUrl = "";
                    return;
                }
                action.QueryParameters = "";
                string[] splitParameters = parameters.Split('/');
                string parameterName;
                string parameterValue;
                //create a well formed querystring based on the array of parameters
                for (int parameterIndex = 0; parameterIndex < splitParameters.Length; parameterIndex++)
                {
                    //ignore the page name 
                    if (splitParameters[parameterIndex].IndexOf(".aspx", StringComparison.InvariantCultureIgnoreCase) == -1 )
                    {
                        //get parameter name
                        parameterName = splitParameters[parameterIndex].Trim();
                        if (parameterName.Length > 0)
                        {                            
                            //rule = GetModuleRule(portalId, CultureCode, TabId, parameterName);
                            rule = null;
                            
                            //add parameter to SendTo if it does not exist already  && not pagename
                            if (action.QueryParameters.IndexOf("?" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1 &&
                                action.QueryParameters.IndexOf("&" + parameterName + "=", StringComparison.InvariantCultureIgnoreCase) == -1 &&
                                (parameterIndex != splitParameters.Length -1 || rule!= null) )
                            {                               
                                //get parameter value
                                parameterValue = "";
                                if (rule != null)
                                {
                                    action.QueryParameters += "&" + rule.Parameters;
                                }
                                else
                                {
                                    action.QueryParameters += "&" + parameterName;
                                    if (parameterIndex < splitParameters.Length - 1)
                                    {
                                        parameterIndex += 1;
                                        if (!String.IsNullOrEmpty(splitParameters[parameterIndex].Trim()))
                                        {
                                            parameterValue = splitParameters[parameterIndex].Trim();
                                            parameterValue = parameterValue.Replace(".aspx", "");

                                        }                                        
                                    }
                                    //add the parameter value
                                    if (parameterValue.Length > 0)
                                    {
                                        action.QueryParameters +=  "=" + parameterValue;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        private static int ProcessParametersWithoutPage(int portalId, RewriterAction action)
        {
            if ((action.WorkUrl.Trim().Length > 0))
            {
                //split the value into an array based on "/" ( ie. /tabid/##/ )
                string parameters = action.WorkUrl.Replace("\\", "/").TrimStart('/');
                var rule = GetModuleRule(portalId, action.CultureCode, parameters);
                if (rule != null)
                {
                    if (rule.Action == UrlRuleAction.Redirect)
                    {
                        /*
                        if (rule.RemoveTab)
                        {
                            redirect.RedirectUrl = redirect.RedirectUrl.Replace("/" + tabUrl, "");
                        }
                         */
                        //action.RedirectUrl = action.RedirectUrl.Replace(parameters.TrimStart('/'), rule.RedirectDestination);
                        action.RedirectModule = rule.RedirectDestination;
                        action.DoRedirect = true;
                        action.Raison += "+ModuleRule:" + rule.Url + ">" + rule.RedirectDestination;

                    }
                    if (rule.TabId != Null.NullInteger)
                    {
                        action.ModuleUrl = parameters;
                        action.ModuleParameters = rule.Parameters;
                        action.TabId = rule.TabId;
                        action.WorkUrl = "";
                    }
                }
                
            }
            return -1;
        }

        private static IEnumerable<UrlRule> GetRules(int portalId)
        {
            if (portalId < 0) // host menu
                return new List<UrlRule>();
            else
                return UrlRuleConfiguration.GetConfig(portalId).Rules;
        }
        /*
        private static IEnumerable<UrlRule> GetRules(int portalId, string CultureCode)
        {
            if (CultureCode == "") CultureCode = null;            
            return GetRules(portalId).Where(r => string.Equals(r.CultureCode,CultureCode, StringComparison.OrdinalIgnoreCase));
        }
        */
        private static UrlRule GetFirstRule(IEnumerable<UrlRule> rules, string CultureCode)
        {
            UrlRule rule = null;
            if (CultureCode == "") CultureCode = null;
            if (CultureCode == null)
            {
                rule = rules.FirstOrDefault(r => string.IsNullOrEmpty(r.CultureCode));
            }
            else
            {
                rule = rules.FirstOrDefault(r => string.Equals(r.CultureCode, CultureCode, StringComparison.OrdinalIgnoreCase));
                if (rule == null) {
                    rule = rules.FirstOrDefault(r => string.IsNullOrEmpty(r.CultureCode));
                }
            }
            return rule;
        }


        private static UrlRule GetModuleRule(int portalId, string CultureCode, int TabId, string Url)
        {
            UrlRule rule = null;
            IEnumerable<UrlRule> rules;
            if (TabId == Null.NullInteger)
            {
                rules = GetRules(portalId).Where(r => r.RuleType == UrlRuleType.Module && r.Url == Url && r.RemoveTab == true);
            }
            else
            {
                rules = GetRules(portalId).Where(r => r.RuleType == UrlRuleType.Module && r.Url == Url );
            }

            if (TabId != Null.NullInteger)
            {
                //with tabid
                var tabRules = rules.Where(r => r.TabId == TabId);
                rule = GetFirstRule(tabRules, CultureCode);
            }
            //without tabid
            if (rule == null)
            {
                rule = GetFirstRule(rules, CultureCode);
            }            
            if (rule == null)
            {
                // redirection rule from Rewrite rule RedirectDestination
                rule = GetRedirectModuleRule(portalId, CultureCode, TabId, Url);
            }
            return rule;
        }

        
        private static UrlRule GetModuleRule(int portalId, string CultureCode, string Url)
        {
            return GetModuleRule(portalId, CultureCode, Null.NullInteger, Url);
        }

        // redirection rule from Rewrite rule RedirectDestination
        private static UrlRule GetRedirectModuleRule(int portalId, string CultureCode, int TabId, string Url)
        {
            Url = Url.ToLower();
            UrlRule rule = null;
            bool RemoveTab = TabId == Null.NullInteger;
            var rules = GetRules(portalId).Where(r => r.RuleType == UrlRuleType.Module && r.RedirectDestination == Url && r.Action == UrlRuleAction.Rewrite /*&& r.RemoveTab == RemoveTab*/);
            //with tabid
            if (TabId != Null.NullInteger)
            {
                var tabRules = rules.Where(r => r.TabId == TabId);
                rule = GetFirstRule(tabRules, CultureCode);
            }
            //without tabid
            if (rule == null)
            {
                rule = GetFirstRule(rules, CultureCode);
            }
            if (rule != null)
            {
                rule = new UrlRule()
                {
                    CultureCode = rule.CultureCode,
                    TabId = rule.TabId,
                    RuleType = rule.RuleType,
                    Action = UrlRuleAction.Redirect,
                    Url = rule.RedirectDestination,
                    RedirectDestination = rule.Url,
                    RemoveTab = rule.RemoveTab                
                };                
            }            
            return rule;
        }
        
        private static UrlRule GetTabRule(int portalId, string CultureCode, string Url)
        {
            // if force to lowercase look at all case bacause we redirect uppercase
            //if (UrlRewiterSettings.IsUrlToLowerCase())
            {
                Url = Url.ToLower();
            }
            
            var rules = GetRules(portalId).Where(r => r.RuleType == UrlRuleType.Tab && r.Url == Url);
            var rule = GetFirstRule(rules, CultureCode);
            return rule;
        }

        private static UrlRule GetRewriteTabRule(int portalId, string CultureCode, int TabId)
        {                                    
            var rules = GetRules(portalId).Where(r => r.RuleType == UrlRuleType.Tab && r.Action == UrlRuleAction.Rewrite && r.TabId == TabId);
            var rule = GetFirstRule(rules, CultureCode);
            return rule;
        }

/*
        private static UrlRule GetTabModuleRule(int portalId, string CultureCode, string Url)
        {
            var rule = GetRules(portalId, CultureCode).SingleOrDefault(r => r.RuleType == UrlRuleType.TabModule && r.Url == Url);
            if (rule == null && !string.IsNullOrEmpty(CultureCode))
            {
                rule = GetRules(portalId, null).SingleOrDefault(r => r.RuleType == UrlRuleType.Tab && r.Url == Url);
            }
            return rule;
        }
*/
        private static UrlRule GetLanguageRule(int portalId, string Url)
        {
            var rule = GetRules(portalId).FirstOrDefault(r => r.RuleType == UrlRuleType.Culture && r.Url == Url);
            return rule;
        }

        public void OnBeginRequest(object s, EventArgs e)
        {
            var app = (HttpApplication) s;
            var server = app.Server;
            var request = app.Request;
            var response = app.Response;
            var requestedPath = app.Request.Url.AbsoluteUri;

            if (RewriterUtils.OmitFromRewriteProcessing(request.Url.LocalPath))
            {
                return;
            }
			
            //'Carry out first time initialization tasks
            Initialize.Init(app);
            if (request.Url.LocalPath.ToLower().EndsWith("/install/install.aspx")
                || request.Url.LocalPath.ToLower().EndsWith("/install/upgradewizard.aspx")
                || request.Url.LocalPath.ToLower().EndsWith("/install/installwizard.aspx")
                || request.Url.LocalPath.ToLower().EndsWith("captcha.aspx")
                || request.Url.LocalPath.ToLower().EndsWith("scriptresource.axd") 
                || request.Url.LocalPath.ToLower().EndsWith("webresource.axd"))
            {
                return;
            }
			
            //URL validation 
            //check for ".." escape characters commonly used by hackers to traverse the folder tree on the server
            //the application should always use the exact relative location of the resource it is requesting
            var strURL = request.Url.AbsolutePath;
            var strDoubleDecodeURL = server.UrlDecode(server.UrlDecode(request.RawUrl));
            if (Regex.Match(strURL, "[\\\\/]\\.\\.[\\\\/]").Success || Regex.Match(strDoubleDecodeURL, "[\\\\/]\\.\\.[\\\\/]").Success)
            {
                DotNetNuke.Services.Exceptions.Exceptions.ProcessHttpException(request);
            }
            try
            {
                //fix for ASP.NET canonicalization issues http://support.microsoft.com/?kbid=887459
                if ((request.Path.IndexOf("\\") >= 0 || Path.GetFullPath(request.PhysicalPath) != request.PhysicalPath))
                {
                    DotNetNuke.Services.Exceptions.Exceptions.ProcessHttpException(request);
                }
            }
            catch (Exception exc)
            {
                //DNN 5479
                //request.physicalPath throws an exception when the path of the request exceeds 248 chars.
                //example to test: http://localhost/dotnetnuke_2/xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx/default.aspx
                DnnLog.Error(exc);

            }


            String domainName;
            RewriteUrl(app, out domainName);

            //blank DomainName indicates RewriteUrl couldn't locate a current portal
            //reprocess url for portal alias if auto add is an option
            if(domainName == "" && CanAutoAddPortalAlias())
            {
                domainName = Globals.GetDomainName(app.Request, true);
            }

            //from this point on we are dealing with a "standard" querystring ( ie. http://www.domain.com/default.aspx?tabid=## )
            //if the portal/url was succesfully identified

            var tabId = Null.NullInteger;
            var portalId = Null.NullInteger;
            string portalAlias = null;
            PortalAliasInfo portalAliasInfo = null;
            var parsingError = false;

            // get TabId from querystring ( this is mandatory for maintaining portal context for child portals )
            if (!string.IsNullOrEmpty(request.QueryString["tabid"]))
            {
                if (!Int32.TryParse(request.QueryString["tabid"], out tabId))
                {
                    tabId = Null.NullInteger;
                    parsingError = true;
                }
            }

            // get PortalId from querystring ( this is used for host menu options as well as child portal navigation )
            if (!string.IsNullOrEmpty(request.QueryString["portalid"]))
            {
                if (!Int32.TryParse(request.QueryString["portalid"], out portalId))
                {
                    portalId = Null.NullInteger;
                    parsingError = true;
                }
            }

            if (parsingError)
            {
                //The tabId or PortalId are incorrectly formatted (potential DOS)
                DotNetNuke.Services.Exceptions.Exceptions.ProcessHttpException(request);
            }


            try
            {
                //alias parameter can be used to switch portals
                if (request.QueryString["alias"] != null)
                {
                    // check if the alias is valid
                    var childAlias = request.QueryString["alias"];
                    if (!Globals.UsePortNumber())
                    {
                        childAlias = childAlias.Replace(":" + request.Url.Port, "");
                    }

                    if (PortalAliasController.GetPortalAliasInfo(childAlias) != null)
                    {
                        //check if the domain name contains the alias
                        if (childAlias.IndexOf(domainName, StringComparison.OrdinalIgnoreCase) == -1)
                        {
                            //redirect to the url defined in the alias
                            response.Redirect(Globals.GetPortalDomainName(childAlias, request, true), true);
                        }
                        else //the alias is the same as the current domain
                        {
                            portalAlias = childAlias;
                        }
                    }
                }
				
				//PortalId identifies a portal when set
                if (portalAlias == null)
                {
                    if (portalId != Null.NullInteger)
                    {
                        portalAlias = PortalAliasController.GetPortalAliasByPortal(portalId, domainName);
                    }
                }
				
				//TabId uniquely identifies a Portal
                if (portalAlias == null)
                {
                    if (tabId != Null.NullInteger)
                    {
                        //get the alias from the tabid, but only if it is for a tab in that domain
                        portalAlias = PortalAliasController.GetPortalAliasByTab(tabId, domainName);
                        if (String.IsNullOrEmpty(portalAlias))
                        {
                            //if the TabId is not for the correct domain
                            //see if the correct domain can be found and redirect it 
                            portalAliasInfo = PortalAliasController.GetPortalAliasInfo(domainName);
                            if (portalAliasInfo != null && !request.Url.LocalPath.ToLower().EndsWith("/linkclick.aspx"))
                            {
                                if (app.Request.Url.AbsoluteUri.StartsWith("https://", StringComparison.InvariantCultureIgnoreCase))
                                {
                                    strURL = "https://" + portalAliasInfo.HTTPAlias.Replace("*.", "");
                                }
                                else
                                {
                                    strURL = "http://" + portalAliasInfo.HTTPAlias.Replace("*.", "");
                                }
                                if (strURL.IndexOf(domainName, StringComparison.InvariantCultureIgnoreCase) == -1)
                                {
                                    strURL += app.Request.Url.PathAndQuery;
                                }
                                response.Redirect(strURL, true);
                            }
                        }
                    }
                }
				
                //else use the domain name
                if (String.IsNullOrEmpty(portalAlias))
                {
                    portalAlias = domainName;
                }
                //using the DomainName above will find that alias that is the domainname portion of the Url
                //ie. dotnetnuke.com will be found even if zzz.dotnetnuke.com was entered on the Url
                portalAliasInfo = PortalAliasController.GetPortalAliasInfo(portalAlias);
                if (portalAliasInfo != null)
                {
                    portalId = portalAliasInfo.PortalID;
                }
				
                //if the portalid is not known
                if (portalId == Null.NullInteger)
                {
                    bool autoAddPortalAlias = CanAutoAddPortalAlias();

                    if (!autoAddPortalAlias && !request.Url.LocalPath.EndsWith(Globals.glbDefaultPage, StringComparison.InvariantCultureIgnoreCase))
                    {
                        // allows requests for aspx pages in custom folder locations to be processed
                        return;
                    }

                    if (autoAddPortalAlias)
                    {
                        var portalAliasController = new PortalAliasController();
                        portalId = Host.HostPortalID;
                        //the domain name was not found so try using the host portal's first alias
                        if (portalId > Null.NullInteger)
                        {
                            portalAliasInfo = new PortalAliasInfo();
                            portalAliasInfo.PortalID = portalId;
                            portalAliasInfo.HTTPAlias = portalAlias;
                            portalAliasController.AddPortalAlias(portalAliasInfo);

                            response.Redirect(app.Request.Url.ToString(), true);
                        }
                    }
                }
            }
            catch (ThreadAbortException exc)
            {
                //Do nothing if Thread is being aborted - there are two response.redirect calls in the Try block
                DnnLog.Debug(exc);

            }
            catch (Exception ex)
            {
				//500 Error - Redirect to ErrorPage
                DnnLog.Error(ex);

                strURL = "~/ErrorPage.aspx?status=500&error=" + server.UrlEncode(ex.Message);
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Server.Transfer(strURL);
            }
            if (portalId != -1)
            {
                //load the PortalSettings into current context
                var portalSettings = new PortalSettings(tabId, portalAliasInfo);
                app.Context.Items.Add("PortalSettings", portalSettings);

                // load PortalSettings and HostSettings dictionaries into current context
                // specifically for use in DotNetNuke.Web.Client, which can't reference DotNetNuke.dll to get settings the normal way
                app.Context.Items.Add("PortalSettingsDictionary", PortalController.GetPortalSettingsDictionary(portalId));
                app.Context.Items.Add("HostSettingsDictionary", HostController.Instance.GetSettingsDictionary());
                
                if (portalSettings.PortalAliasMappingMode == PortalSettings.PortalAliasMapping.Redirect && 
                    !String.IsNullOrEmpty(portalSettings.DefaultPortalAlias) &&
                    portalAliasInfo != null &&
                    portalAliasInfo.HTTPAlias != portalSettings.DefaultPortalAlias)
                {
                    //Permanently Redirect
                    response.StatusCode = 301;
                    
                    var redirectAlias = Globals.AddHTTP(portalSettings.DefaultPortalAlias);
                    var checkAlias = Globals.AddHTTP(portalAliasInfo.HTTPAlias);
                    var redirectUrl = redirectAlias + request.RawUrl;
                    if(redirectUrl.StartsWith(checkAlias, StringComparison.InvariantCultureIgnoreCase))
                    {
                        redirectUrl = redirectAlias + redirectUrl.Substring(checkAlias.Length);
                    }

                    response.AppendHeader("Location", redirectUrl);
                }

                //manage page URL redirects - that reach here because they bypass the built-in navigation
                //ie Spiders, saved favorites, hand-crafted urls etc
                if (!String.IsNullOrEmpty(portalSettings.ActiveTab.Url) && request.QueryString["ctl"] == null && request.QueryString["fileticket"] == null)
                {
					//Target Url
                    var redirectUrl = portalSettings.ActiveTab.FullUrl;
                    if (portalSettings.ActiveTab.PermanentRedirect)
                    {
						//Permanently Redirect
                        response.StatusCode = 301;
                        response.AppendHeader("Location", redirectUrl);
                    }
                    else
                    {
						//Normal Redirect
                        response.Redirect(redirectUrl, true);
                    }
                }
				
                //manage secure connections
                if (request.Url.AbsolutePath.EndsWith(".aspx", StringComparison.InvariantCultureIgnoreCase))
                {
					//request is for a standard page
                    strURL = "";
                    //if SSL is enabled
                    if (portalSettings.SSLEnabled)
                    {
						//if page is secure and connection is not secure orelse ssloffload is enabled and server value exists
                        if ((portalSettings.ActiveTab.IsSecure && !request.IsSecureConnection) && (IsSSLOffloadEnabled(request) == false))
                        {
							//switch to secure connection
                            strURL = requestedPath.Replace("http://", "https://");
                            strURL = FormatDomain(strURL, portalSettings.STDURL, portalSettings.SSLURL);
                        }
                    }
                    //if SSL is enforced
                    if (portalSettings.SSLEnforced)
                    {
						//if page is not secure and connection is secure 
                        if ((!portalSettings.ActiveTab.IsSecure && request.IsSecureConnection) )
                        {
                            //check if connection has already been forced to secure orelse ssloffload is disabled
                            if (request.QueryString["ssl"] == null)
                            {
                                strURL = requestedPath.Replace("https://", "http://");
                                strURL = FormatDomain(strURL, portalSettings.SSLURL, portalSettings.STDURL);
                            }
                        }
                    }
					
					//if a protocol switch is necessary
                    if (!String.IsNullOrEmpty(strURL))
                    {
                        if (strURL.StartsWith("https://", StringComparison.InvariantCultureIgnoreCase))
                        {
							//redirect to secure connection
                            response.Redirect(strURL, true);
                        }
                        else //when switching to an unsecure page, use a clientside redirector to avoid the browser security warning
                        {
                            response.Clear();
                            //add a refresh header to the response 
                            response.AddHeader("Refresh", "0;URL=" + strURL);
                            //add the clientside javascript redirection script
                            response.Write("<html><head><title></title>");
                            response.Write("<!-- <script language=\"javascript\">window.location.replace(\"" + strURL + "\")</script> -->");
                            response.Write("</head><body></body></html>");
                            //send the response
                            response.End();
                        }
                    }
                }
            }
            else
            {
                //alias does not exist in database
                //and all attempts to find another have failed
                //this should only happen if the HostPortal does not have any aliases
                //404 Error - Redirect to ErrorPage
                strURL = "~/ErrorPage.aspx?status=404&error=" + domainName;
                HttpContext.Current.Response.Clear();
                HttpContext.Current.Server.Transfer(strURL);
            }

            if (app.Context.Items["FirstRequest"] != null)
            {
                app.Context.Items.Remove("FirstRequest");

                //Process any messages in the EventQueue for the Application_Start_FirstRequest event
                EventQueueController.ProcessMessages("Application_Start_FirstRequest");
            }
        }


       

        private bool IsSSLOffloadEnabled(HttpRequest request)
        {

            var ssloffloadheader = HostController.Instance.GetString("SSLOffloadHeader", "");
            //if the ssloffloadheader variable has been set check to see if a request header with that type exists
            if (!string.IsNullOrEmpty(ssloffloadheader.ToString()))
            {
                var ssloffload = request.Headers[ssloffloadheader.ToString()];
                if (!string.IsNullOrEmpty(ssloffload))
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }
            return false;
        }

        private bool CanAutoAddPortalAlias()
        {
            var autoAddPortalAlias = HostController.Instance.GetBoolean("AutoAddPortalAlias");
            autoAddPortalAlias = autoAddPortalAlias && (new PortalController().GetPortals().Count == 1);
            return autoAddPortalAlias;
        }

        
 

    }

    class RewriterAction {

        public RewriterAction() {
            TabId = Null.NullInteger;
            CultureCode = null;
        }

        public string LocalPath { get; set; }
        public String OriginalUrl { get; set; }
        public string HostPort { get; set; }
        public string Alias { get; set; }

        // Url parts
        public string CultureUrl { get; set; }
        public string PageUrl { get; set; }
        public string ModuleUrl { get; set; }
        public string QueryUrl { get; set; }

        public string WorkUrl { get; set; }

        // Rewrite information
        public string CultureCode { get; set; }
        public int TabId { get; set; }
        public string ModuleParameters { get; set; }
        public string QueryParameters { get; set; }

        // redirection information
        public String RedirectCulture { get; set; }
        public String RedirectPage { get; set; }
        public String RedirectModule { get; set; }

        //public String RedirectUrl { get; set; }
        public int Status { get; set; }
        public bool DoRedirect { get; set; }
        public string Raison { get; set; }
        public bool RedirectHomePage { get; set; }
    }

}