﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Collections.Generic;
using System.Text;
using DotNetNuke.Entities.Portals;


namespace Satrabel.HttpModules.Provider
{
    /// <summary>
    /// Description résumée de UrlProvider
    /// </summary>
    public abstract class UrlRuleProvider
    {
        protected UrlRuleProvider(){}
        
        public string Name {
            get {
                return this.GetType().Name;            
            }
        }
        public string HelpUrl { get; set; }

        public abstract List<UrlRule> GetRules(int PortalId);

        public string[] CacheKeys { get; set; }

        public UrlRuleSetting[] Settings { get; set; }

        public bool GetPortalSettingAsBoolean(int portalID, string key)
        {
            bool DefaultValue = false;
            UrlRuleSetting set = Settings.SingleOrDefault(s => s.Name == key);
            if (set != null) 
            {
                DefaultValue = bool.Parse(set.DefaultValue);
            }
            return PortalController.GetPortalSettingAsBoolean( Name + "_"+ key, portalID, DefaultValue);
        }
       
        protected static string CleanupUrl(string Url)
        {
            string replaceWith = "-";

            string AccentFrom = "ÀÁÂÃÄÅàáâãäåÒÓÔÕÖØòóôõöøÈÉÊËèéêëÌÍÎÏìíîïÙÚÛÜùúûüÿÑñÇç";
            string AccentTo   = "AAAAAAaaaaaaOOOOOOooooooEEEEeeeeIIIIiiiiUUUUuuuuyNnCc";

            Url = Url.ToLower();

            StringBuilder result = new StringBuilder(Url.Length);
            string ch = ""; int i = 0; int last = Url.ToCharArray().GetUpperBound(0);
            foreach (char c in Url.ToCharArray())
            {

                //use string for manipulation
                ch = c.ToString();
                if (ch == " ")
                {
                    ch = replaceWith;
                }
                else if (@".[]|:;`%\\""".Contains(ch))
                    ch = "";
                else if (@" &$+,/=?@~#<>()¿¡«»!'’".Contains(ch))
                    ch = replaceWith;
                else
                {
                    for (int ii = 0; ii < AccentFrom.Length; ii++)
                    {
                        if (ch == AccentFrom[ii].ToString())
                        {
                            ch = AccentTo[ii].ToString();
                        }
                    }
                }

                if (i == last)
                {
                    if (!(ch == "-" || ch == replaceWith))
                    {   //only append if not the same as the replacement character
                        result.Append(ch);
                    }
                }
                else
                    result.Append(ch);
                i++;//increment counter
            }
            result = result.Replace(replaceWith + replaceWith, replaceWith);
            result = result.Replace(replaceWith + replaceWith, replaceWith);
            return result.ToString();
        }
    }

    public enum UrlRuleSettingType
    {
        Boolen = 1
    }

    public class UrlRuleSetting {

        string _Name;
        string _DefaultValue;
        UrlRuleSettingType _ValueType;

        public UrlRuleSetting (string Name, bool DefaultValue)
        {
            _Name = Name;
            _DefaultValue = DefaultValue.ToString();
            _ValueType = UrlRuleSettingType.Boolen;        
        }

        public string Name { get { return _Name; } }
        public string DefaultValue { get { return _DefaultValue; } }
        public UrlRuleSettingType ValueType { get { return _ValueType; } }        
    }

}