﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Collections;
using System.Collections.Generic;
using DotNetNuke.Common.Utilities;

using Ventrian.SimpleGallery;
using DotNetNuke.Entities.Modules;
using DotNetNuke.Framework.Providers;
using Ventrian.SimpleGallery.Entities;
using Ventrian.SimpleGallery.Common;
using Satrabel.HttpModules.Provider;

namespace Satrabel.OpenUrlRewriter.SimpleGallery
{
    public class SimpleGalleryUrlRuleProvider : UrlRuleProvider
    {

        private const string ProviderType = "urlRule";
        private const string ProviderName = "simpleGalleryUrlRuleProvider";

        private readonly ProviderConfiguration _providerConfiguration = ProviderConfiguration.GetProviderConfiguration(ProviderType);
        private readonly bool includePageName = true;

        public SimpleGalleryUrlRuleProvider() {
            var objProvider = (DotNetNuke.Framework.Providers.Provider)_providerConfiguration.Providers[ProviderName];
            if (!String.IsNullOrEmpty(objProvider.Attributes["includePageName"]))
            {
                includePageName = bool.Parse(objProvider.Attributes["includePageName"]);
            }
            //CacheKeys = new string[] { "SimpleGallery-ProperyValues-All" };
        }

        public override List<UrlRule> GetRules(int PortalId)
        {
            List<UrlRule> Rules = new List<UrlRule>();

            AlbumController ac = new AlbumController();
            ModuleController mc = new ModuleController();
            ArrayList modules = mc.GetModulesByDefinition(PortalId, "SimpleGallery");
            //foreach (ModuleInfo module in modules.OfType<ModuleInfo>().GroupBy(m => m.ModuleID).Select(g => g.First())){                
            foreach (ModuleInfo module in modules.OfType<ModuleInfo>())
            {
                var albums = ac.List(module.ModuleID, Null.NullInteger, true, true, AlbumSortType.CreateDate, Ventrian.SimpleGallery.Common.SortDirection.ASC);
                foreach (AlbumInfo album in albums)
                {
                    var rule = new UrlRule
                    {
                        CultureCode = module.CultureCode,
                        TabId = module.TabID,
                        RuleType = UrlRuleType.Module,
                        Parameters = "AlbumID="+album.ModuleID.ToString()+"-"+album.AlbumID.ToString(),
                        Action = UrlRuleAction.Rewrite,
                        Url = CleanupUrl(album.Caption),
                        RemoveTab = !includePageName
                    };
                    AlbumInfo parentAlbum = album;
                    while (parentAlbum.ParentAlbumID != Null.NullInteger)
                    {
                        parentAlbum = ac.Get(parentAlbum.ParentAlbumID);
                        rule.Url = CleanupUrl(CleanupUrl(parentAlbum.Caption)) + "/" + rule.Url;
                    }                    
                    Rules.Add(rule);
                    
                }
            }
            return Rules;
        }

    }
}